/******************************************************************************
    QtAV:  Media play library based on Qt and FFmpeg
    Copyright (C) 2012-2014 Wang Bin <wbsecg1@gmail.com>

*   This file is part of QtAV

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************/

#include "QtAV/AudioOutputTypes.h"
#include "QtAV/FactoryDefine.h"
#include "QtAV/factory.h"

namespace QtAV {

AudioOutputId AudioOutputId_PortAudio = 1;
AudioOutputId AudioOutputId_OpenAL = 2;
AudioOutputId AudioOutputId_OpenSL = 3;

QVector<AudioOutputId> GetRegistedAudioOutputIds()
{
    return QVector<AudioOutputId>::fromStdVector(AudioOutputFactory::registeredIds());
}


FACTORY_DEFINE(AudioOutput)

extern void RegisterAudioOutputPortAudio_Man();
extern void RegisterAudioOutputOpenAL_Man();
extern void RegisterAudioOutputOpenSL_Man();

void AudioOutput_RegisterAll()
{
#if QTAV_HAVE(PORTAUDIO)
    RegisterAudioOutputPortAudio_Man();
#endif //QTAV_HAVE(PORTAUDIO)
#if QTAV_HAVE(OPENAL)
    RegisterAudioOutputOpenAL_Man();
#endif //QTAV_HAVE(OPENAL)
#if QTAV_HAVE(OPENSL)
    RegisterAudioOutputOpenSL_Man();
#endif //QTAV_HAVE(OPENSL)
}

} //namespace QtAV
